XIncludeFile "vsm.pbi"

#DebugMode = #False ; #True

Enumeration   ;  .
  #Status_Init ;    .
  #Status_Cmd  ;       .
  #Status_Proc ;   .
  #Status_Send ;  .
EndEnumeration

Enumeration 2 ;    allback().
  #CB_Init     ;  .
  #CB_St_s     ;     .
  #CB_St_0     ;   .
  #CB_St_1     ;   .
  #CB_SendInfo ;    .
  #CB_SendInfoEnd
EndEnumeration

Enumeration
  #Type_DHT11
  #Type_DHT22
EndEnumeration

Structure DataBuff
  crc.a
  Thermo.w
  Humidity.u
EndStructure

Structure MinMax_Info
  Min.w
  Max.w
EndStructure

Structure DelayParam
  StartSignal.q  ; 20 .
  WaitResponse.q ; 30 .
  OutResponse.q  ; 80 .
  DelayTransmitBit.q ; 50 .
  DelaySendData_0.q ; 24 .
  DelaySendData_1.q ; 72 .
  ConversionTime.q ; 1 
EndStructure

Macro mClassPrivate
  ;    IACTIVEMODEL.
  *component.ICOMPONENT
  *instance.IINSTANCE
  
  ;    IDSIMMODEL.
  *inst.IINSTANCE
  *ckt.IDSIMCKT
  *MyPopup.IDEBUGPOPUP
  ;  .
  *Data.IDSIMPIN ;   .
  
  OldTime.ABSTIME
  OldTime_Info.ABSTIME ;     .
  
  ModelType.a ;   (DHT11, DHT22  . .).
  
  Status.a ;   .   #Status_xxx.
  
  OutPin_State.a ; #True    . 1   #False  . 0.
  NumBit.a       ;   .
  
  StartStatus.a
  MouseInit.a
  Actuate_Count.l
    
  Thermo.w
  Humidity.u
  Steps.u
  
  Th.MinMax_Info
  Hum.MinMax_Info
  Delay.DelayParam
  
  StructureUnion
    OutBuff.DataBuff
    OutQ.q
  EndStructureUnion
EndMacro

Structure Class_IDSIM
  *vt_IDSIM ;pointer to virtual table as first entry.
  
  mClassPrivate ;  .
EndStructure 

Structure Class_IACTIVE
  *vt_IACTIVE ;pointer to virtual table as first entry.
  *vt_IDSIM
  
  mClassPrivate ;  .
EndStructure 

EnableExplicit
EnableASM

Declare ModelNEW(device.s)
Declare animate(element.i, *newstate.ACTIVEDATA)

ProcedureCDLL.i createactivemodel(device.s, *ils.ILICENCESERVER)
  Protected *Result=0
  
  If *ils\Authorize($80808081)
    *Result=ModelNEW(device.s) ;  .
  EndIf
  
  ProcedureReturn *Result
EndProcedure

ProcedureCDLL deleteactivemodel(*model.Class_IACTIVE)
  If *model
    
    If *model\ModelType = #Type_DHT11
      *model\component\addprop("Th", StrF(*model\Thermo, 0), #HIDE_ALL)
      *model\component\addprop("Hum", StrF(*model\Humidity, 0), #HIDE_ALL)
    ElseIf *model\ModelType = #Type_DHT22
      *model\component\addprop("Th", StrF(*model\Thermo/10, 1), #HIDE_ALL)
      *model\component\addprop("Hum", StrF(*model\Humidity/10, 1), #HIDE_ALL)
    EndIf
    
    ;ClearStructure(*model, Class_IACTIVE)
    FreeMemory(*model) ;  ,  .
  EndIf
EndProcedure


Procedure ModelNEW(device.s)   ;  .
  Protected *Class.Class_IACTIVE=0
  
  *Class = AllocateMemory(SizeOf(Class_IACTIVE)) 
  If *Class
    
    ;InitializeStructure(*Class, Class_IACTIVE)
    
    *Class\vt_IACTIVE = ?IACTIVE_Funct ;   IACTIVEMODEL.
    *Class\vt_IDSIM   = ?IDS_Funct     ;   IDSIMMODEL.
    *Class\Status = #Status_Init
    
    Select UCase(device)
      Case "DHT11"
        *Class\ModelType = #Type_DHT11
      Case "DHT22"
        *Class\ModelType = #Type_DHT22
      Default
        FreeMemory(*Class)
        *Class=0
    EndSelect
    
  EndIf   
  
  ProcedureReturn *Class 
EndProcedure

Procedure.d GetDoubleParam(*This.Class_IDSIM, Name.s, Def.d)
  Protected Result.d
  
  *This\inst\getnumval(@Result, Name, Def)
  
  If *This\ModelType=#Type_DHT22
    Result*10
  EndIf
  
  ProcedureReturn Result
EndProcedure

Procedure ResData(*This.Class_IDSIM, Time.ABSTIME) ;        .
  Protected i, CRC.a=0
  
  *This\OldTime_Info=Time
  
  If *This\ModelType=#Type_DHT11
    *This\OutBuff\Thermo=0
    *This\OutBuff\Humidity=0
    PokeA(@*This\OutBuff\Thermo+1, *This\Thermo)
    PokeA(@*This\OutBuff\Humidity+1, *This\Humidity)
  ElseIf *This\ModelType=#Type_DHT22
    *This\OutBuff\Thermo=*This\Thermo ; .
    *This\OutBuff\Humidity=*This\Humidity ; .
  EndIf
  
  If *This\OutBuff\Thermo & $8000 ;  .
    *This\OutBuff\Thermo ! $7FFF
  EndIf
  
  For i=1 To 4
    CRC + PeekA(@*This\OutQ+i)
  Next i
  *This\OutBuff\CRC=CRC
  
EndProcedure

CompilerIf #DebugMode = #True
Procedure Dbg(*This.Class_IDSIM, String.s)
  String+#CRLF$
  If *This And *This\MyPopup And @*This\MyPopup\Print()
    CallCFunctionFast(@*This\MyPopup\Print(), *This\MyPopup, @String, 0)
  EndIf
EndProcedure
CompilerEndIf

;-   
;{

;- IACTIVEMODEL.

;  .
;        IComponent.
Procedure Initialize(*cpt.ICOMPONENT)
  Protected *This.Class_IACTIVE, s.s, c.l, i
  MOV *This, ecx
  
  *This\component = *cpt
  *This\component\createtextstyle(#Null)
  *This\component\settextfont("Arial") 
  *This\component\setbold(#True)
  *This\component\settextsize(100)
  *This\component\settextcolour($00FF00)
  
EndProcedure

; ISIS   ,    .
;      .
; ,       "PRIMITIVE=.ANALOG.ammeter",
;      "AMMETER".
Procedure.i GetSpiceModel(*sPrimitive)
  Protected *This.Class_IACTIVE
  MOV *This, ecx
  
  ProcedureReturn 0 ;    .
EndProcedure

;   ,  ISIS   ,
;    .
; ,     "PRIMITIVE=.DIGITAL.display",
;       "DISPLAY".
Procedure.i GetDsimModel(*sPrimitive)
  Protected *This.Class_IACTIVE
  MOV *This, ecx
  
  ProcedureReturn *This+OffsetOf(Class_IACTIVE\vt_IDSIM) ;    IDSIMMODEL.
EndProcedure

;   ,      ().
;     animate()     ,
;  ,   .
;  ,       .
;    : *This\component\drawstate(state).
Procedure Plot_IACTIVE(state.ACTIVESTATE)
  Protected *This.Class_IACTIVE
  MOV *This, ecx
  
  *This\component\drawstate(state)
  MOV ecx, *This ; Thiscall .
  animate(0, 0)  ;     .
   
EndProcedure

;     ,    ,
; ( ISPICEMODEL  IDSIMMODEL)    .
;     PROSPICE  
;    IDSIMMODEL\indicate(),
;      .
;    ,   
;        .
;    ,    
;  RTVPROBE, RTIPROBE  RTDPROBE  ,   
; ,       .
;  "element" -   ,    .
;       RTxPROBE,
;      ,     .
;  *newstate    .
Procedure Animate(element.i, *newstate.ACTIVEDATA)
  Protected *This.Class_IACTIVE, Thermo.s, Humidity.s
  MOV *This, ecx
  
  If *This\ModelType=#Type_DHT11
    Thermo=StrF(*This\Thermo, 0)
    Humidity=StrF(*This\Humidity, 0)
  ElseIf *This\ModelType=#Type_DHT22
    Thermo=StrF(*This\Thermo/10, 1)
    Humidity=StrF(*This\Humidity/10, 1)
  EndIf
  
  *This\component\drawsymbol(1)
  
  CallCFunctionFast(@*This\component\DrawText(), *This\component, 630, -150, 0, #TXJ_RIGHT|#TXJ_MIDDLE, @Thermo)
  CallCFunctionFast(@*This\component\DrawText(), *This\component, 630, -350, 0, #TXJ_RIGHT|#TXJ_MIDDLE, @Humidity)
  
EndProcedure

;       ,
;         .
;          ().
;     #True,  ,   "" 
;       ,    #False.
;  : key -    Windows,
;    -        ,
; flags -    : 1 - , 2 - .
Procedure.b Actuate_IACTIVE(Key.w, x.i, y.i, Flags.l)
  Protected *This.Class_IACTIVE, Result = #False
  MOV *This, ecx
  
    
  If Flags & #ACF_LEFT ;    
    
    *This\Actuate_Count+1
    If *This\Actuate_Count%4=0 Or *This\Actuate_Count=1 Or *This\Actuate_Count>20
      
      If (x>=700 And x<=800) And (y<=-100 And y>=-200) ;  .
        *This\Thermo+*This\Steps
        If *This\Thermo>*This\Th\Max
          *This\Thermo=*This\Th\Max
        Else
          Result = #True
        EndIf
      ElseIf (x>=850 And x<=950) And (y<=-100 And y>=-200) ;  .
        *This\Thermo-*This\Steps
        If *This\Thermo<*This\Th\Min
          *This\Thermo=*This\Th\Min
        Else
          Result = #True
        EndIf
      EndIf
      
      If (x>=700 And x<=800) And (y<=-310 And y>=-410) ;  .
        *This\Humidity+*This\Steps
        If *This\Humidity>*This\Hum\Max
          *This\Humidity=*This\Hum\Max
        Else
          Result = #True
        EndIf
      ElseIf (x>=850 And x<=950) And (y<=-310 And y>=-410) ;  .
        *This\Humidity-*This\Steps
        If *This\Humidity<*This\Hum\Min
          *This\Humidity=*This\Hum\Min
        Else
          Result = #True
        EndIf
      EndIf
      
        
    Else
      Result = #True
    EndIf
    
  Else
    *This\Actuate_Count=0
  EndIf
    
  
  If *This\MouseInit<>#True
    *This\MouseInit=#True
    Result = #True
  EndIf
    
  ;Dbg(*This+OffsetOf(Class_IACTIVE\vt_IDSIM), "x="+x+"    y="+y+"    f="+flags)
  
  ProcedureReturn Result
EndProcedure


;- IDSIMMODEL.

;  isdigital.
;    1,       ,  0,  .
;     ,  ,     1.
Procedure Isdigital(*PinName)
  Protected *This.Class_IDSIM
  MOV *This, ecx
  
  ProcedureReturn 1 ;     .
EndProcedure

;  setup.
;     IINSTANCE   ,
;  IDSIMCKT   . 
;        , 
;        IINSTANCE  IDSIMCKT.
; ,         
; (   *This\inst\getdsimpin).
Procedure Setup(*Instance.IINSTANCE, *Dsimckt.IDSIMCKT)
  Protected *This.Class_IDSIM, td.d, mult=0
  MOV *This, ecx

  
  *This\inst = *Instance ;    .
  *This\ckt  = *Dsimckt
  
  *This\Data = *This\inst\getdsimpin("Data", #True)
  *This\Data\setstate2(0, 1, #FLT) ;   .
  *This\OldTime_Info=0

  *This\Thermo=GetDoubleParam(*This, "Th", 24) ; .
  *This\Humidity=GetDoubleParam(*This, "Hum", 54)  ; .
  *This\Th\Min=GetDoubleParam(*This, "T_MIN", -40)
  *This\Th\Max=GetDoubleParam(*This, "T_MAX", 80)
  *This\Hum\Min=GetDoubleParam(*This, "H_MIN", 0)
  *This\Hum\Max=GetDoubleParam(*This, "H_MAX", 99.9)
  *This\Steps=GetDoubleParam(*This, "Step", 0.1)
  
    
  If *This\Thermo<*This\Th\Min
    *This\Thermo=*This\Th\Min
  ElseIf *This\Thermo>*This\Th\Max
    *This\Thermo=*This\Th\Max
  EndIf
  
  If *This\Humidity<*This\Hum\Min
    *This\Humidity=*This\Hum\Min
  ElseIf *This\Humidity>*This\Hum\Max
    *This\Humidity=*This\Hum\Max
  EndIf
  
  If *This\Steps<=0
    *This\Steps=1
  EndIf
  
  ;   !
  *This\Delay\StartSignal = *This\inst\getinitval("DSS", 16000) * 1000000
  *This\Delay\WaitResponse = *This\inst\getinitval("DWR", 30) * 1000000
  *This\Delay\OutResponse = *This\inst\getinitval("DOR", 80) * 1000000
  *This\Delay\DelayTransmitBit = *This\inst\getinitval("DTB", 50) * 1000000
  *This\Delay\DelaySendData_0 = *This\inst\getinitval("DS0", 24) * 1000000
  *This\Delay\DelaySendData_1 = *This\inst\getinitval("DS1", 72) * 1000000
  *This\Delay\ConversionTime = *This\inst\getinitval("CT", 1000) * 1000000000
  
  CompilerIf #DebugMode = #True
  Protected cps.CREATEPOPUPSTRUCT
  cps\caption = "Debugger Log"  ;   
  cps\flags  = #PWF_VISIBLE| ;   
               #PWF_SIZEABLE ;     .
  cps\type   = #PWT_DEBUG    ;   - DEBUG 
  cps\height = 500           ;   
  cps\width  = 300           ;   
  cps\id     = 200           ;   
  *This\MyPopup = *instance\createpopup(@cps)
  CompilerEndIf
  
  ;   ,       (  ).
  *This\ckt\setcallback(*This\Delay\ConversionTime, *This, #CB_Init)
EndProcedure

;  runctl       ,
;     .
;    ,    
;     .
Procedure Runctrl(Mode.RUNMODES)
  Protected *This.Class_IDSIM
  MOV *This, ecx
  
EndProcedure

;  actuate ,         
;   .      /  /. 
Procedure Actuate_IDS(Time.REALTIME, NewState.ACTIVESTATE)
  Protected *This.Class_IDSIM
  MOV *This, ecx
  
EndProcedure

;  indicate      .
;    FALSE,       .
Procedure Indicate(Time.REALTIME,  *aData.ACTIVEDATA)
  Protected *This.Class_IDSIM
  MOV *This, ecx
  
  ProcedureReturn #False ;#True
EndProcedure

;  simulate      .
;            ,
;     ,   -     .
Procedure Simulate(Time.ABSTIME, Mode.DSIMMODES)
  Protected *This.Class_IDSIM
  MOV *This, ecx
  
  If *This\Status = #Status_Cmd ;     .
    
    If *This\Data\isedge() ;   .
      
      If *This\Data\isnegedge() ;   (  1  0).
        *This\StartStatus = #False
        *This\OldTime = Time
      Else                      ;   (  0  1).
        If Time-*This\OldTime>=*This\Delay\StartSignal ; 16 .
          *This\StartStatus = #True
          *This\ckt\setcallback(Time+*This\Delay\WaitResponse, *This, #CB_St_s) ; 30 .
        EndIf
      EndIf
      
    EndIf
    
  EndIf
  
EndProcedure

;  callback     ,
;    *This\ckt\setcallback(). Ÿ     .
Procedure CallBack(Time.ABSTIME, EventID.EVENTID)
  Protected *This.Class_IDSIM
  MOV *This, ecx
    
  Select EventID 
    Case #CB_Init
      *This\Status = #Status_Cmd ;     .
    Case #CB_St_s
      If *This\StartStatus = #True
        *This\Data\setstate2(Time, 2, #SLO) ;    .
        *This\ckt\setcallback(Time+*This\Delay\OutResponse, *This, #CB_St_0) ; 80 .
        ResData(*This, Time) ;        .
        *This\Status = #Status_Proc
      EndIf
    Case #CB_St_0
      *This\Data\setstate2(Time, 2, #FLT) ;   .
      *This\ckt\setcallback(Time+*This\Delay\OutResponse, *This, #CB_St_1) ; 80 .
    Case #CB_St_1
      *This\Status = #Status_Send
      *This\NumBit = 0
      *This\OutPin_State = #False
      *This\Data\setstate2(Time, 2, #SLO) ;    .
      *This\ckt\setcallback(Time+*This\Delay\DelayTransmitBit, *This, #CB_SendInfo) ; 50 .
    Case #CB_SendInfo
      If *This\Status = #Status_Send
        If *This\OutPin_State = #False ;  .
          If *This\NumBit<40
            *This\OutPin_State = #True
            *This\Data\setstate2(Time, 2, #FLT) ;   .
            If (*This\OutQ << ((*This\NumBit+24))) & $8000000000000000
              *This\ckt\setcallback(Time+*This\Delay\DelaySendData_1, *This, #CB_SendInfo) ; 70 .
            Else
              *This\ckt\setcallback(Time+*This\Delay\DelaySendData_0, *This, #CB_SendInfo) ; 28 .
            EndIf
            *This\NumBit+1
          Else
            *This\Data\setstate2(Time, 2, #FLT) ;   .
            *This\ckt\setcallback(Time+*This\Delay\ConversionTime, *This, #CB_SendInfoEnd) ;    .
          EndIf
        Else                           ; . 0.
          *This\OutPin_State = #False
          *This\Data\setstate2(Time, 2, #SLO) ;    .
          *This\ckt\setcallback(Time+*This\Delay\DelayTransmitBit, *This, #CB_SendInfo) ; 50 .
        EndIf
      EndIf
      
    Case #CB_SendInfoEnd
      *This\Status = #Status_Cmd ;     .
  EndSelect
  
EndProcedure

DataSection
  IACTIVE_Funct:
  Data.i @Initialize()
  Data.i @GetSpiceModel()
  Data.i @GetDsimModel()
  Data.i @Plot_IACTIVE()
  Data.i @Animate()
  Data.i @Actuate_IACTIVE()
  IDS_Funct:
  Data.i @Isdigital()
  Data.i @Setup()
  Data.i @Runctrl()
  Data.i @Actuate_IDS()
  Data.i @Indicate()
  Data.i @Simulate()
  Data.i @CallBack()
EndDataSection

;}
; IDE Options = PureBasic 5.24 LTS (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 283
; FirstLine = 271
; Folding = ----
; EnableAsm
; Executable = DHT.dll
; IncludeVersionInfo
; VersionField0 = 1.2.0.0
; VersionField1 = 1.2.0.0
; VersionField2 = ϸ
; VersionField3 = DHT Sensor Model
; VersionField4 = 1.2
; VersionField5 = 1.2
; VersionField6 =   DHT11  DHT22  Proteus.
; VersionField7 = DHT Model
; VersionField8 = %EXECUTABLE
; VersionField13 = pure-basic@yandex.ru
; VersionField14 = http://pure-basic.narod.ru/
; VersionField16 = VFT_DLL